@@ -1,5 +1,5 @@
 from argparse import ArgumentParser
-from typing import List, Tuple, Optional
+from typing import List, Tuple, Optional, Dict
 
 import numpy as np
 from numpy.random import RandomState
@@ -8,12 +8,14 @@ from rlai.actions import Action
 from rlai.environments import Environment
 from rlai.environments.mdp import ModelBasedMdpEnvironment
 from rlai.meta import rl_text
+from rlai.models.feature_extraction import FeatureExtractor
+from rlai.q_S_A.function_approximation.models.feature_extraction import (
+    StateActionInteractionFeatureExtractor
+)
 from rlai.rewards import Reward
 from rlai.states.mdp import MdpState
 from rlai.utils import parse_arguments
-from rlai.value_estimation.function_approximation.models.feature_extraction import (
-    StateActionInteractionFeatureExtractor
-)
+from rlai.v_S.function_approximation.models.feature_extraction import StateFeatureExtractor
 
 
 @rl_text(chapter=3, page=60)
@@ -240,7 +242,11 @@ class GridworldFeatureExtractor(StateActionInteractionFeatureExtractor):
 
         parsed_args, unparsed_args = parse_arguments(cls, args)
 
-        fex = GridworldFeatureExtractor(
+        # there shouldn't be anything left
+        if len(vars(parsed_args)) > 0:  # pragma no cover
+            raise ValueError('Parsed args remain. Need to pass to constructor.')
+
+        fex = cls(
             environment=environment
         )
 
@@ -250,14 +256,14 @@ class GridworldFeatureExtractor(StateActionInteractionFeatureExtractor):
             self,
             states: List[MdpState],
             actions: List[Action],
-            for_fitting: bool
+            refit_scaler: bool
     ) -> np.ndarray:
         """
         Extract features for state-action pairs.
 
         :param states: States.
         :param actions: Actions.
-        :param for_fitting: Whether the extracted features will be used for fitting (True) or prediction (False).
+        :param refit_scaler: Whether or not to refit the feature scaler before scaling the extracted features.
         :return: State-feature numpy.ndarray.
         """
 
@@ -281,16 +287,19 @@ class GridworldFeatureExtractor(StateActionInteractionFeatureExtractor):
             actions=actions
         )
 
-    def get_feature_names(
+    def get_action_feature_names(
             self
-    ) -> List[str]:
+    ) -> Dict[str, List[str]]:
         """
-        Get names of extracted features.
+        Get names of actions and their associated feature names.
 
-        :return: List of feature names.
+        :return: Dictionary of action names and their associated feature names.
         """
 
-        return ['from-top', 'from-bottom', 'from-left', 'from-right']
+        return {
+            a.name: ['from-top', 'from-bottom', 'from-left', 'from-right']
+            for a in self.actions
+        }
 
     def __init__(
             self,
@@ -314,3 +323,107 @@ class GridworldFeatureExtractor(StateActionInteractionFeatureExtractor):
 
         self.num_rows = environment.grid.shape[0]
         self.num_cols = environment.grid.shape[1]
+
+
+@rl_text(chapter='Feature Extractors', page=1)
+class GridworldStateFeatureExtractor(StateFeatureExtractor):
+    """
+    A feature extractor for the gridworld. This extractor does not interact feature values with actions. Its primary use
+    is in state-value estimation (e.g., for the baseline of policy gradient methods).
+    """
+
+    @classmethod
+    def get_argument_parser(
+            cls
+    ) -> ArgumentParser:
+        """
+        Get argument parser.
+
+        :return: Argument parser.
+        """
+
+        parser = ArgumentParser(
+            prog=f'{cls.__module__}.{cls.__name__}',
+            parents=[super().get_argument_parser()],
+            allow_abbrev=False,
+            add_help=False
+        )
+
+        return parser
+
+    @classmethod
+    def init_from_arguments(
+            cls,
+            args: List[str],
+            environment: Gridworld
+    ) -> Tuple[FeatureExtractor, List[str]]:
+        """
+        Initialize a feature extractor from arguments.
+
+        :param args: Arguments.
+        :param environment: Environment.
+        :return: 2-tuple of a feature extractor and a list of unparsed arguments.
+        """
+
+        parsed_args, unparsed_args = parse_arguments(cls, args)
+
+        # there shouldn't be anything left
+        if len(vars(parsed_args)) > 0:  # pragma no cover
+            raise ValueError('Parsed args remain. Need to pass to constructor.')
+
+        fex = cls(
+            environment=environment
+        )
+
+        return fex, unparsed_args
+
+    def extract(
+            self,
+            state: MdpState,
+            refit_scaler: bool
+    ) -> np.ndarray:
+        """
+        Extract state features.
+
+        :param state: State.
+        :param refit_scaler: Unused.
+        :return: State-feature vector.
+        """
+
+        row = int(state.i / self.num_cols)
+        col = state.i % self.num_cols
+
+        state_features = np.array([
+            row,  # from top
+            self.num_rows - row - 1,  # from bottom
+            col,  # from left
+            self.num_cols - col - 1  # from right
+        ])
+
+        return state_features
+
+    def get_feature_names(
+            self
+    ) -> List[str]:
+        """
+        Get names of features.
+
+        :return: List of feature names.
+        """
+
+        return ['from-top', 'from-bottom', 'from-left', 'from-right']
+
+    def __init__(
+            self,
+            environment: Gridworld
+    ):
+        """
+        Initialize the feature extractor.
+
+        :param environment: Environment.
+        """
+
+        super().__init__()
+
+        self.num_rows = environment.grid.shape[0]
+        self.num_cols = environment.grid.shape[1]
